
function discoverAirboxes(){
    /*
      This function scans all devices in zone0 at the property tree
      and retrieves those with EAN number matching WESCO_AIRBOX_EAN.

      The associated airbox variables at the property tree are
      retrieved/initialized.
    */

	function ConfigurationIsValid(path){
	    // Returns true if these fields exist
	    try{
		if(getFromPropertyTree_nostore(path+"/dSID",0,false)==undefined){
		    return false;
		}
		if(getFromPropertyTree_nostore(path+"/Level_1",0,false)==undefined){
		    return false;
		}
		if(getFromPropertyTree_nostore(path+"/Level_2",0,false)==undefined){
		    return false;
		}
		if(getFromPropertyTree_nostore(path+"/Level_3",0,false)==undefined){
		    return false;
		}
		if(getFromPropertyTree_nostore(path+"/errorCode",0,false)==undefined){
		    return false;
		}
		if(getFromPropertyTree_nostore(path+"/errorTime",0,false)==undefined){
		    return false;
		}
		if(getFromPropertyTree_nostore(path+"/errorCounter",0,false)==undefined){
		    return false;
		}
		if(getFromPropertyTree_nostore(path+"/errorFSMState",0,false)==undefined){
		    return false;
		}
		return true;
	    }
	    catch (err){
		debug("DEBUG","Configuration Invalid: "+ path+";error->"+err);
		return false;
	    }
	} // end 	function ConfigurationIsValid(path){


	function removeNonPresentDevices(){
	    /*This function recursevely removes from the property tree
	     * devices that are not anymore present.  This function
	     * does not yet consider subfolders (only one level per
	     * airbox).
	     */

	    var airboxesPath="Airboxes/Devices";

	    try{
		var prop=Property.getNode(airboxesPath);
		var array = prop.getChildren();  // vector of airboxes indexes, empty if the node does not exist


		for (var i in array){ // iterate over the airboxes nodes
		    var AirboxName=array[i].getName();

		    var AirboxPath=array[i].getPath();
		    if(!ConfigurationIsValid(AirboxPath)){ // if the configuration is not valid
			var subprop=Property.getNode(AirboxPath);
			var subarray=subprop.getChildren();

			// iterate over all airbox properties and delete them
			for (var j in subarray){
			    var item=subarray[j].getName();
			    subprop.removeChild(item);
			}
			// once all the variables were removed, remove the
			// airbox node.
			prop.removeChild(AirboxName);

			debug("DEBUG","Removed device:"+AirboxPath);
		    }else{

			var AirboxID=getFromPropertyTree(AirboxPath+"/dSID",0,false);
			var currentAirbox=getDevices().byDSID(AirboxID);

			if(!currentAirbox.isPresent){ // if device is not present
			    var subprop=Property.getNode(AirboxPath);
			    var subarray=subprop.getChildren();

			    // iterate over all airbox properties and delete them
			    for (var j in subarray){
				var item=subarray[j].getName();
				subprop.removeChild(item);
			    }
			    // once all the variables were removed, remove the
			    // airbox node.
			    prop.removeChild(AirboxName);

			    WESCOLog("W",ErrorCodes["deleted_from_propTree"],AirboxID,"");
			    debug("DEBUG","Removed device:"+AirboxID);
			}
		    }
		}
	    }
	    catch(err){
		debug("catch "+err);
		//do nothing. For instance if the directory Devices does not yet exist
	    }
	}// end ...     function removeNonPresentDevices(){



    function createDefaultAirbox(dsuid,id){
	// Initializes/retrieves variables corresponding to each
	// airbox.

	var path="Airboxes/Devices/"+dsuid+"/"
	saveToPropertyTree_nostore(path+"dSID",id,true);
	getFromPropertyTree_nostore(path+"Level_1",AIRBOX_MINIMUM_LEVEL_DEFAULT,true);
	getFromPropertyTree_nostore(path+"Level_2",AIRBOX_MEDIUM_LEVEL_DEFAULT,true);
	getFromPropertyTree_nostore(path+"Level_3",AIRBOX_NOMINAL_LEVEL_DEFAULT,true);
	getFromPropertyTree_nostore(path+"errorCode",ErrorCodes["no_error"],true);
	getFromPropertyTree_nostore(path+"errorTime",DEFAULT_STRING,true);
	getFromPropertyTree_nostore(path+"errorCounter",0,true);
	getFromPropertyTree_nostore(path+"errorFSMState","ready",true);
	//	Property.store();

    } // end ...    function createDefaultAirbox(dsuid,id){



    removeNonPresentDevices(); // remove entries of non present devices


    // the list of airboxes is created based on the devices in zone0,
    // whose EAN matches the Airbox one.
    var Nairboxes=0;

    getDevices().perform(function (device) {

	var path="/apartment/zones/zone0/devices/"+device.dsuid+"/"
	var EAN=getFromPropertyTree(path+"productInfo/EAN",false,false);
	if (EAN==WESCO_AIRBOX_EAN){
	    Nairboxes=Nairboxes+1;
	}

	if (device.isPresent){
	    var dSID=getFromPropertyTree_nostore(path+"dSID",false,false);

	    // if the EAN number matches the WESCO one, then create an airbox node.
	    if (EAN==WESCO_AIRBOX_EAN){
		debug("DEBUG","Create airbox node;EAN:"+EAN+";dSID:"+dSID+";dsuid:"+device.dsuid+";Name:"+device.name);

		// check if the airbox is already at the property tree
		var AirboxPath="Airboxes/Devices/"+device.dsuid;
		var prop=Property.getNode(AirboxPath);

		if (prop===undefined || prop===null){
		    WESCOLog("I",ErrorCodes["registered_to_propTree"],dSID,"");
		}
		createDefaultAirbox(device.dsuid,dSID);
	    }
	}
    });


    // makes the uninstallable flag conditional. The app can be uninstall if there are not airboxes present.
    if (Nairboxes > 0){
	saveToPropertyTree_nostore("system/preventUninstall",true,true);
	debug("DEBUG","making up uninstallable");
    }else{
	saveToPropertyTree_nostore("system/preventUninstall",false,true);
	debug("DEBUG","making up non-uninstallable");
    }

    Property.store();
    debug("DEBUG","Completed Airbox discovery");
}


function isAirbox(dsuid){
    /*
      This function verifies wheather a dsuid belongs to an airbox or not.
      The function does not verify if the device is present or not.
    */

    var AirboxPath="Airboxes/Devices/"+dsuid;
    var prop=Property.getNode(AirboxPath);

    if(prop){
	return true;
    }else{
	return false;
    }
}


function airboxSendSceneFSM(FSMState,Device,Dict,Scene){
    /*
      This function implements the FSM on charge of sending a Scene
      value to each one of the airboxes involved in the control.

      Scene is the corresponding level to which all airboxes will be
      set symmetrically

      Dict is a dictionary with a list of booleans (Regs), one per each
      register. Each boolean indicates when the corresponding register
      was already successfully written. The dictionary also contains
      Ntries.  Ntries: is the cummulative number of writing attempts.
    */


    var NextFSMState;
    var NextDict;
    var NextDevice;

    // arm the corresponding timer
    var timer=timers["TIMER_sendSceneFSM"];
    var parameter={};


    function initializeAirboxDict(){

	/* Initializes the airbox dictionary. This dictionary has each
	 * airbox dSID, a counter of reading tries, and a boolean
	 * indicating whether the airbox level was already
	 * successfully written or not.
	 */

	var airboxesPath="Airboxes/Devices";
	var prop=Property.getNode(airboxesPath);
	var array = prop.getChildren();  // vector of airboxes indexes, empty if the node does not exist

	var Dict={};

	for (var i in array){
	    var AirboxPath=array[i].getPath();
	    var AirboxID=getFromPropertyTree(AirboxPath+"/dSID",0,false);
	    Dict[i]={};
	    Dict[i]={"dSID":AirboxID,"Path":AirboxPath,"Ntries":0,"completed":false};
	}

	return Dict;
    }

    function cloneDict(Dict){
	// clone object Dict and return the copy
	var NewDict={};
	for (prop in Dict){
	    NewDict[prop]=Dict[prop];
	}
	return NewDict;
    }


    //debug("DEBUG","Write Dict "+JSON.stringify(Dict)+";state:"+FSMState);
    debug("DEBUG","state:"+FSMState);

    //FSM: this state machine writes the levels
    switch(FSMState){

    case "initialize":
	NextDict=initializeAirboxDict();
	NextFSMState="writing";
	NextDevice=Device;

	parameter['Dict']=JSON.stringify(NextDict);
	parameter['Device']=NextDevice;
	parameter['FSMState']=NextFSMState;
	parameter['Scene']=Scene;

	callTimer(timer.name,timer.id,timer.duration,parameter);

	break;

    case "writing":

	var AirboxID=Dict[String(Device)].dSID;
	var AirboxPath=Dict[String(Device)].Path;
	NextDict=cloneDict(Dict);

	if(!Dict[String(Device)].completed) { // if the register hasn't been written
	    try{
		// write LEVEL register
		var currentAirbox=getDevices().byDSID(AirboxID);
		if(currentAirbox.isPresent){
		    currentAirbox.setConfig(DSTINY_REG_BANK,AIRBOX_LEVEL_REG,parseInt(Scene));
		    debug("DEBUG","writing_airbox:"+AirboxID+";register:1;level:"+Scene);
		    NextDict[String(Device)].Ntries++; // increment the counter of readings
		    if (NextDict[String(Device)].Ntries > 1 ){ // Added to write each airbox twice
			NextDict[String(Device)].completed=true; // mark register as ALREADY WRITTEN
		    }
		}else{
		    NextDict[String(Device)].completed=true; // mark register as ALREADY WRITTEN
		    NextDict[String(Device)].Ntries++; // increment the counter of readings
		}
	    }
	    catch(err){
		//WESCOLog("W",ErrorCodes["timeout_error"],AirboxID,"");
		debug("DEBUG","exception;"+err+";"+AirboxID);
		NextDict[String(Device)].Ntries++; // increment the counter of readings
	    }
	}


	// Exit condition: The exit condition is when all registers
	// have been written or the max number of tries exceeded

	var AllDone=true;
	var MaxTries=false;

	for (var Dev in Dict){
	    AllDone=AllDone && Dict[String(Dev)].completed;
	    MaxTries = MaxTries || (Dict[String(Dev)].Ntries >= AIRBOX_WRITE_MAX_RETRY);

	    //If an airbox can not be reached.  No communication!!
	    if (Dict[String(Dev)].Ntries >= AIRBOX_WRITE_MAX_RETRY){
		var dsid=Dict[String(Dev)].dSID;
		WESCOLog("E",ErrorCodes["timeout_error"],dsid,"");
	    }
	}

	if (AllDone || MaxTries){ //exit condition
	    NextFSMState="done";
	    NextDevice=100; // an invalid airbox device index
	}else{
	    NextDevice=Device+1;
	    if (!(NextDevice in Dict)){
		NextDevice=0;
	    }
	    //if not yet done ... continue writing
	    NextFSMState="writing";
	}

	parameter['Dict']=JSON.stringify(NextDict);
	parameter['Device']=NextDevice;
	parameter['FSMState']=NextFSMState;
	parameter['Scene']=Scene;

	callTimer(timer.name,timer.id,timer.duration,parameter);
	break;
    default:
	NextFSMState="done";
	//unlockScript();
	break;
    }//end switch
} //end ...  function airboxSendSceneFSM(FSMState,Device,Dict,Scene){


function AirboxErrorFSM(dsuid,ErrorCode,Caller){
    /*
      This FSM implements the error handling for a particular airbox defined by its dsuid.

      The second parameter is the ErrorCode as received from the
      airbox Status Event.  The ErrorCode is only relevant at the ready state.

      Caller identifies the source  that has invoked the FSM.  Caller can be "errorEvent" or "Timer".

      The FSM has three states:

      ready: if an error arrives, checks based on the date of the
      previous occurrence wheather it is the first occurrence or not.
      Some errors are reported immediately.

      The state machine attempts to remove some errors. If the number
      of tries is larger than AIRBOX_CLEARERROR_NTRIES -1, then the error is
      reported.

      The error removal consist of setting and unsetting a flag at the airbox, which
      is done in the next two states,  clearing_on and clearing_off
      respectively.

      This FSM is only called if the variable
      Airboxes/Devices/dsuid/errorCode is undefined or is equal to
      NOERROR.  This variable has to be reset (to NOERROR) from the
      UI (see function clearAllAirboxErrors).


    */


    function nextFSMState(AirboxPath,state){
	// writes the next state
	saveToPropertyTree(AirboxPath+"/errorFSMState",state,true);
    }

    function logError(ErrorCode,AirboxID,msg){

	//Writes the corresponding error code to the log file.
	//Returns the code.

	//Error decoding: status bit structure:

	// [7] Fehler
	// [6] Serviceanforderung (nicht definiert)
	// [5] Frostwarnung
	// [4] Frostgefahr / Frostfehler
	// [3] Klappenfehler
	// [2] Ventilatorfehler
	// [1] Sensorfehler
	// [0] Kein Fehler

	var error=parseInt(ErrorCode);
	var errorString=ErrorCodes["no_error"];

	if (error == 128){
	    errorString=ErrorCodes["power_error"];
	    WESCOLog("E",errorString,AirboxID,msg);
	}
	if(error & (1<<1)){
	    errorString=ErrorCodes["sensor_error"];
	    WESCOLog("E",errorString,AirboxID,msg);
	}
	if(error & (1<<2)){
	    errorString=ErrorCodes["fan_error"];
	    WESCOLog("E",errorString,AirboxID,msg);
	}
	if(error & (1<<3)){
	    errorString=ErrorCodes["flap_error"];
	    WESCOLog("E",errorString,AirboxID,msg);
	}

	if(error & (1<<5)){
	    errorString=ErrorCodes["frost_warning"];
	    WESCOLog("W",errorString,AirboxID,msg);
	}

	if(error & (1<<4)){
	    errorString=ErrorCodes["frost_error"]; // deprecated
	    WESCOLog("E",errorString,AirboxID,msg);
	}

	return errorString;
    }//end ... function logError(ErrorCode,AirboxID,msg){


    var ErrorStr;
    var AirboxPath="Airboxes/Devices/"+dsuid;
    var dsid=getFromPropertyTree(AirboxPath+"/dSID",false,false);
    var dnow=new Date(); // current date

    // arms the corresponding timer for the different FSM transitions.
    var timer=timers["TIMER_clearupAirboxError"];
    timer.id=timer.id+dsuid; //create a per-device timer
    var parameter={"dsuid":dsuid};  // this dictionary is passed to the timer call function



    // gets the current FSM state from the property tree [ready,clearing_on,clearing_off]
    var errorFSMState=getFromPropertyTree(AirboxPath+"/errorFSMState","ready",true);


    switch(errorFSMState){
    case "ready":
	debug("DEBUG","case ready;dsuid:"+dsuid);
	switch(parseInt(ErrorCode)){

	case AIRBOX_SENSOR_ERR:  case AIRBOX_FLAP_ERR: // sensor error  and flap error

	    // No error removal attempt for those errors.  They are reported as soon as they appear.


	    ErrorStr=logError(ErrorCode,dsid,"No error removal attempt");
	    saveToPropertyTree_nostore(AirboxPath+"/errorCode",ErrorStr,true);
	    saveToPropertyTree_nostore(AirboxPath+"/errorTime",dnow.toISOString(),true);
	    Property.store();
	    closeExhaustFlap();

	    debug("DEBUG","sending notification");
	    //------------------------------------------
	    // Notification
	    sendPushNotification("Error");
	    debug("DEBUG","sending email");
	    sendEmailError(ErrorStr,dsid);

	    break;

	case AIRBOX_POWER_ERR: case AIRBOX_FAN_ERR: case AIRBOX_FROST_ERR: // power error, fan error  and sensor error

	    // for those errors the error removal function is carried out

	    // time of previous error occurrence
	    var dprev=getFromPropertyTree(AirboxPath+"/errorTime",dnow.toISOString(),true);

	    if (dprev===undefined||dprev==DEFAULT_STRING){
		dprev=new Date(0);
	    }else{
		dprev=new Date(dprev);
	    }


	    // number of times the error has occurred
	    var errorCounter=parseInt(getFromPropertyTree(AirboxPath+"/errorCounter",0,false));
	    if(errorCounter===undefined || errorCounter==DEFAULT_INT){
		errorCounter=0;
	    }

	    //elapsed time in seconds between the present and the previous error occurrences
	    var DeltaTime=parseInt(dnow-dprev)/1000; // in  seconds


	    // if the elapsed time < AIRBOX_MAX_TIME_BTW_ERRORS increase the errorCounter
	    if (DeltaTime < AIRBOX_MAX_TIME_BTW_ERRORS){
		if (errorCounter==0){
		    logError(ErrorCode,dsid,"First occurrence");
		}
		errorCounter++;

	    }else{
		// if the elapsed time is larger or equal to
		// AIRBOX_MAX_TIME_BTW_ERRORS then reset the counter
		// and increment. Log the first error occurrence
		logError(ErrorCode,dsid,"First occurrence");
		errorCounter=1;// restart to zero and increase
	    }

	    closeExhaustFlap();

	    // The error removal is triggered only a limited number of
	    // times. After that, no more attempts are carried out and
	    // the error is reported. Timer durations are defined in timer.js

	    if (errorCounter < AIRBOX_CLEARERROR_NTRIES){
		if (parseInt(ErrorCode)==AIRBOX_POWER_ERR){ // power_error
		    callTimer(timer.name,timer.id,POWER_ERROR_CLEAR_WAIT,parameter);
		}else if (parseInt(ErrorCode)==AIRBOX_FAN_ERR) {
		    callTimer(timer.name,timer.id,FAN_ERROR_CLEAR_WAIT,parameter);
		}
		else {
		    callTimer(timer.name,timer.id,FROST_ERROR_CLEAR_WAIT,parameter);
		}

		// restart the ntries counter, how many times one has
		// tried to write the error removal command
		saveToPropertyTree(AirboxPath+"/clearErrorNtries",0,true);

		// next FSM state: goes to set the clearing error function
		nextFSMState(AirboxPath,"clearing_on");

	    }else{
		// if the error could not be removed ...
		//log the error
		debug("DEBUG","reporting error");
		ErrorStr=logError(ErrorCode,dsid,"Error removal failed");
		//writes the error code  to the property tree
		saveToPropertyTree(AirboxPath+"/errorCode",ErrorStr,true);

		nextFSMState(AirboxPath,"ready");

		debug("DEBUG","sending notification");
		//------------------------------------------
		// Notification
		sendPushNotification("Error");
		debug("DEBUG","sending email");
		sendEmailError(ErrorStr,dsid);
		//------------------------------------------

	    }
	    saveToPropertyTree_nostore(AirboxPath+"/errorTime",dnow.toISOString(),true);
	    saveToPropertyTree_nostore(AirboxPath+"/errorCounter",errorCounter,true);
	    Property.store();
	    break;
	default:
	    break;
	} // end ..	switch(parseInt(ErrorCode)){


	break;  //end ... case "ready":

    case "clearing_on":

	if (Caller=="errorEvent"){
	    // When a new error arrives and calls the FSM with
	    // Caller=errorEvent, the FSM processes the error only if
	    // it is at the ready state. Otherwise, it means that the
	    // FSM is busy.
	    break;
	}

	debug("DEBUG","clearing_on;dsuid:"+dsuid);
	// writes the airbox's special command register to remove
	// the error.  After a few seconds this register has to be
	// written back  to 0, this is done in the clearing_off state.
	// Writing the register back to 0 is not necessary in the airbox firmaware version 3.1
	try{
	    AirboxCommandReg(dsuid,2); //write special register to 2 (clear up error)
	    nextFSMState(AirboxPath,"clearing_on_2");
	    callTimer(timer.name,timer.id,"+7",parameter);
	}
	catch(err){
	    var clearErrorNtries=parseInt(getFromPropertyTree(AirboxPath+"/clearErrorNtries",0,true));
	    clearErrorNtries++;

	    if(clearErrorNtries > AIRBOX_WRITE_MAX_RETRY){
		nextFSMState(AirboxPath,"ready");
		clearErrorNtries=0;

		// notify error
		WESCOLog("E",ErrorCodes["timeout_error"],dsid,"");
		debug("DEBUG","Airbox unreachable:"+dsuid);
	    }else{
		nextFSMState(AirboxPath,"clearing_on");
		callTimer(timer.name,timer.id,"+60",parameter); //wait before the next attempt [seconds]
	    }
	    saveToPropertyTree(AirboxPath+"/clearErrorNtries",clearErrorNtries,true);
	}
	break;

    case "clearing_on_2":

	if (Caller=="errorEvent"){
	    // When a new error arrives and calls the FSM with
	    // Caller=errorEvent, the FSM processes the error only if
	    // it is at the ready state. Otherwise, it means that the
	    // FSM is busy.
	    break;
	}

	debug("DEBUG","clearing_on_2;dsuid:"+dsuid);
	// writes the airbox's special command register to remove
	// the error.  After a few seconds this register has to be
	// written back  to 0, this is done in the clearing_off state.
	// Writing the register back to 0 is not necessary in the airbox firmaware version 3.1
	try{
	    AirboxCommandReg(dsuid,2); //write special register to 2 (clear up error)
	    nextFSMState(AirboxPath,"clearing_off");
	    callTimer(timer.name,timer.id,"+7",parameter);
	}
	catch(err){
	    var clearErrorNtries=parseInt(getFromPropertyTree(AirboxPath+"/clearErrorNtries",0,true));
	    clearErrorNtries++;

	    if(clearErrorNtries > AIRBOX_WRITE_MAX_RETRY){
		nextFSMState(AirboxPath,"ready");
		clearErrorNtries=0;

		// notify error
		WESCOLog("E",ErrorCodes["timeout_error"],dsid,"");
		debug("DEBUG","Airbox unreachable:"+dsuid);
	    }else{
		nextFSMState(AirboxPath,"clearing_on_2");
		callTimer(timer.name,timer.id,"+60",parameter); //wait before the next attempt [seconds]
	    }
	    saveToPropertyTree(AirboxPath+"/clearErrorNtries",clearErrorNtries,true);
	}
	break;


    case "clearing_off":
	// this state is not necessary for the airbox firmware version >= 3.1.

	if (Caller=="errorEvent"){
	    // When a new error arrives, the FSM processes it only if
	    // it is at the ready state. Otherwise, it means that the
	    // FSM is busy.
	    break;
	}

	debug("DEBUG","clearing_off;dsuid:"+dsuid);

	// writes back the clearing error flag to 0, by writing the
	// airbox's special command register to 0.
	try{
	    AirboxCommandReg(dsuid,0);
	    nextFSMState(AirboxPath,"clearing_off_2");
	    callTimer(timer.name,timer.id,"+7",parameter);
	    // Write all airboxes to the default value or the last value before the error
	}
	catch(err){
	    var clearErrorNtries=parseInt(getFromPropertyTree(AirboxPath+"/clearErrorNtries",0,true));
	    clearErrorNtries++;

	    if(clearErrorNtries > AIRBOX_WRITE_MAX_RETRY){
		// when completed reinitializes the FSM
		nextFSMState(AirboxPath,"ready");
		clearErrorNtries=0;
		//errorCounter=0; // initializes the error counter
		// notify error
		WESCOLog("E",ErrorCodes["timeout_error"],dsid,"");
		debug("DEBUG","Airbox unreachable:"+dsuid);
	    }else{
		nextFSMState(AirboxPath,"clearing_off");
		callTimer(timer.name,timer.id,"+60",parameter); //wait before the next attempt [seconds]
	    }
	    saveToPropertyTree(AirboxPath+"/clearErrorNtries",clearErrorNtries,true);
	    //saveToPropertyTree(AirboxPath+"/errorCounter",errorCounter,true);
	}
	break;

    case "clearing_off_2":
	// this state is not necessary for the airbox firmware version >= 3.1.

	if (Caller=="errorEvent"){
	    // When a new error arrives, the FSM processes it only if
	    // it is at the ready state. Otherwise, it means that the
	    // FSM is busy.
	    break;
	}

	debug("DEBUG","clearing_off_2;dsuid:"+dsuid);

	// writes back the clearing error flag to 0, by writing the
	// airbox's special command register to 0.
	try{
	    AirboxCommandReg(dsuid,0);
	    nextFSMState(AirboxPath,"ready");
	    setAirboxesToLastCalledScene();
	    debug("DEBUG","set back current level");
	    openExhaustFlap();
	    // Write all airboxes to the default value or the last value before the error

	}
	catch(err){
	    var clearErrorNtries=parseInt(getFromPropertyTree(AirboxPath+"/clearErrorNtries",0,true));
	    clearErrorNtries++;

	    if(clearErrorNtries > AIRBOX_WRITE_MAX_RETRY){
		// when completed reinitializes the FSM
		nextFSMState(AirboxPath,"ready");
		clearErrorNtries=0;
		//errorCounter=0; // initializes the error counter
		// notify error
		WESCOLog("E",ErrorCodes["timeout_error"],dsid,"");
		debug("DEBUG","Airbox unreachable:"+dsuid);
	    }else{
		nextFSMState(AirboxPath,"clearing_off_2");
		callTimer(timer.name,timer.id,"+60",parameter); //wait before the next attempt [seconds]
	    }
	    saveToPropertyTree(AirboxPath+"/clearErrorNtries",clearErrorNtries,true);
	    //saveToPropertyTree(AirboxPath+"/errorCounter",errorCounter,true);
	}
	break;


    default:
	break;

    }//end ... switch(errorFSMState){
} // end ... function  AirboxErrorRoutine(dsuid,ErrorCode){








function AirboxCommandReg(dsuid,value){
    /* Writes airbox's special command register (register 3) to the
     * given value.

     Only some values are allowed:

     -> 2 error clear up
     -> 0
     -> 4 saves parameters to flash
     -> 1 restores default parameters

    */

    var AirboxPath="Airboxes/Devices/"+dsuid;
    var AirboxID=getFromPropertyTree(AirboxPath+"/dSID",0,false);

    switch(value){
    case 0:case 1:case 2: case 4:

	getDevices().byDSID(AirboxID).setConfig(DSTINY_REG_BANK,AIRBOX_COMMAND_REG,parseInt(value));
	break;
    default:
	break;
    }

}


function clearAllAirboxErrors(){
    var airboxesPath="Airboxes/Devices";
    var prop=Property.getNode(airboxesPath);
    var array = prop.getChildren();  // vector of airboxes indexes, empty if the node does not exist


    for (var i in array){
	var AirboxPath=array[i].getPath();
	saveToPropertyTree_nostore(AirboxPath+"/errorCode",ErrorCodes["no_error"],true);
	saveToPropertyTree_nostore(AirboxPath+"/errorCounter",0,true);
	saveToPropertyTree_nostore(AirboxPath+"/errorFSMState","ready",true);
	Property.store();
	debug("DEBUG","Reset error code of airbox "+AirboxPath);
    }
}


function closeExhaustFlap(){
    // closes the exhaust flap whenever one of the airboxes is in error
    var dSID=getFromPropertyTree("ExhaustFlap",'',false);

    if (dSID!==undefined && dSID != DEFAULT_STRING){
	try{
	    getDevices().byDSID(dSID).callScene(13); // dS minimum scene: 13
	}
	catch(err){
	    //report error. Device unreachable
	}
	debug("DEBUG","closing flap:"+dSID);
    }
}

function openExhaustFlap(){

    // opens the exhaust flap whenever none of the airboxes are in error.

    // verifies that other airbox devices are not in error state before opening the exahust flap.
    var noerror=true;

    var airboxesPath="Airboxes/Devices";
    var prop=Property.getNode(airboxesPath);
    var array = prop.getChildren();  // vector of airboxes indexes, empty if the node does not exist

    for (var i in array){
	var AirboxPath=array[i].getPath();
	var errorCode=getFromPropertyTree(AirboxPath+"/errorCode",0,false);
	var ID=getFromPropertyTree(AirboxPath+"/dSID",0,false);
	debug("DEBUG","Airbox:"+ID+";errorCode:"+errorCode);
	noerror=noerror && (errorCode===undefined || errorCode==ErrorCodes["no_error"]);
    }

    if (noerror){
	open();
    }

    function open(){
	var dSID=getFromPropertyTree("ExhaustFlap",'',false);

	if (dSID!==undefined && dSID != DEFAULT_STRING){
	    try{
		getDevices().byDSID(dSID).callScene(14); // dS maximum scene: 14
	    }
	    catch(err){
		//report error. Device unreachable
	    }
	    debug("DEBUG","opening flap:"+dSID);
	}
    }
}

function checkAirboxFilterReplacement(){
    /*This function returns true if the airbox filter has to be replaced*/

    var dateReplacement=getFromPropertyTree("Airboxes/LastFilterReplacementDate",DEFAULT_STRING,false);
    var Today=new Date(); // current date

    if (dateReplacement!==undefined && dateReplacement!=DEFAULT_STRING){
	dateReplacement=new Date(dateReplacement);
	try{
	    var Ndays=parseInt(getFromPropertyTree("Airboxes/FilterReplacementInterval",0,false));
	    var DeltaTime=parseInt(Today-dateReplacement)/(1000*86400); // in  days

	    if(DeltaTime >= Ndays){
		debug("DEBUG","Ndays:"+Ndays+";Deltatime:"+DeltaTime);
		saveToPropertyTree("Airboxes/LastFilterReplacementDate",Today.toISOString(),true); // reset date
		return true;
	    }else{
		return false;
	    }
	}catch(err){
	    // error in the conversion or format
	    return false;
	}

    }else{
	return false;
    }

}

function setAirboxesToLastCalledScene(){
    /*This function restitutes the airboxes to the last called scene*/

    var lastLevel=getFromPropertyTree("Controller/CurrentLevel",DEFAULT_INT,false);
    var Dict={};
    if (lastLevel!==undefined && lastLevel!==null){
	if(lastLevel >=0 && lastLevel<=4){
	    airboxSendSceneFSM("initialize",0,Dict,parseInt(lastLevel+2));
	}
    }

}



function airbox_configureWatchdogFSM(FSMState,Device,Dict){
    /*
      This function configures the airboxes' dstiny watchdog.
    */


    var NextFSMState;
    var NextDict;
    var NextDevice;

    // arm the corresponding timer
    var timer=timers["TIMER_configureWatchdogFSM"];
    var parameter={};


    function initializeAirboxDict(){

	/* Initializes the airbox dictionary. This dictionary has each
	 * airbox dSID, a counter of reading tries, and a boolean
	 * indicating whether the airbox level was already
	 * successfully written or not.
	 */

	var airboxesPath="Airboxes/Devices";
	var prop=Property.getNode(airboxesPath);
	var array = prop.getChildren();  // vector of airboxes indexes, empty if the node does not exist

	var Dict={};

	for (var i in array){
	    var AirboxPath=array[i].getPath();
	    var AirboxID=getFromPropertyTree(AirboxPath+"/dSID",0,false);
	    Dict[i]={};
	    Dict[i]={"dSID":AirboxID,"Path":AirboxPath,"Ntries":0,"completed":false};
	}

	return Dict;
    }

    function cloneDict(Dict){
	// clone object Dict and return the copy
	var NewDict={};
	for (prop in Dict){
	    NewDict[prop]=Dict[prop];
	}
	return NewDict;
    }


    //debug("DEBUG","Write Dict "+JSON.stringify(Dict)+";state:"+FSMState);
    debug("DEBUG","watchdog writing state:"+FSMState);

    //FSM: this state machine writes the levels
    switch(FSMState){

    case "initialize":
	NextDict=initializeAirboxDict();
	NextFSMState="writing";
	NextDevice=Device;

	parameter['Dict']=JSON.stringify(NextDict);
	parameter['Device']=NextDevice;
	parameter['FSMState']=NextFSMState;

	callTimer(timer.name,timer.id,timer.duration,parameter);

	break;

    case "writing":

	var AirboxID=Dict[String(Device)].dSID;
	var AirboxPath=Dict[String(Device)].Path;
	NextDict=cloneDict(Dict);

	if(!Dict[String(Device)].completed) { // if the register hasn't been written
	    try{
		// write LEVEL register
		var currentAirbox=getDevices().byDSID(AirboxID);
		if(currentAirbox.isPresent){
		    currentAirbox.setConfig(0x03,0x3A,30); // write bank 3, register 0x3A to 30seconds
		    debug("DEBUG","writing_airbox's:"+AirboxID+";register:0x3-0x3A;level:30");
		    NextDict[String(Device)].Ntries++; // increment the counter of readings
		    NextDict[String(Device)].completed=true; // mark register as ALREADY WRITTEN
		}else{
		    NextDict[String(Device)].completed=true; // mark register as ALREADY WRITTEN
		    NextDict[String(Device)].Ntries++; // increment the counter of readings
		}
	    }
	    catch(err){
		//WESCOLog("W",ErrorCodes["timeout_error"],AirboxID,"");
		debug("DEBUG","exception;"+err+";"+AirboxID);
		NextDict[String(Device)].Ntries++; // increment the counter of readings
	    }
	}


	// Exit condition: The exit condition is when all registers
	// have been written or the max number of tries exceeded

	var AllDone=true;
	var MaxTries=false;

	for (var Dev in Dict){
	    AllDone=AllDone && Dict[String(Dev)].completed;
	    MaxTries = MaxTries || (Dict[String(Dev)].Ntries >= AIRBOX_WRITE_MAX_RETRY);

	    //If an airbox can not be reached.  No communication!!
	    if (Dict[String(Dev)].Ntries >= AIRBOX_WRITE_MAX_RETRY){
		var dsid=Dict[String(Dev)].dSID;
		WESCOLog("E",ErrorCodes["timeout_error"],dsid,"");
	    }
	}

	if (AllDone || MaxTries){ //exit condition
	    NextFSMState="done";
	    NextDevice=100; // an invalid airbox device index
	}else{
	    NextDevice=Device+1;
	    if (!(NextDevice in Dict)){
		NextDevice=0;
	    }
	    //if not yet done ... continue writing
	    NextFSMState="writing";
	}

	parameter['Dict']=JSON.stringify(NextDict);
	parameter['Device']=NextDevice;
	parameter['FSMState']=NextFSMState;

	callTimer(timer.name,timer.id,timer.duration,parameter);
	break;
    default:
	NextFSMState="done";
	//unlockScript();
	break;
    }//end switch
} //end ...  function airbox_configureWatchdogFSM(FSMState,Device,Dict,Scene)
