/*
  airbox-dss.js
  Angular service that provides airbox-addon-specific functions
*/

var exhooddss = angular.module('exhoodDSS', ['dSS']);


exhooddss.service('exhooddss',['$q','dss',function($q,dss) {
    var rootpath = "/scripts/wesco";
    var service = this;

    var scenenumbers = [0,5,17,18,19];

    // When reading stringified objects (JSON) from the property tree, string has to be decoded before calling JSON.parse
    function HTMLDecode(string) {
	return string.replace(/&amp;/g,'&').replace(/&gt;/g,'>').replace(/&lt;/g,'<').replace(/&quot;/g,'"');
    }




    this.setLevel = function(level) {
	return dss.Event.raise('wesco_UI',{
	    'action':'SetAirboxesLevel','level':level,
	});
	//return dss.Apartment.callScene(scenenumbers[level],groupid);
    };

    this.rediscoverAirboxes = function() {
	return dss.Event.raise('wesco_UI',{
	    'action':'RediscoverAirboxes',
	});
    };


    this.refreshPushTargetOptions = function() {
	return dss.Event.raise('wesco_UI',{
	    'action':'RefreshPushTargetOptions',
	});
    };

    this.readRegs = function(dsid,regs) {
	return dss.Event.raise('wesco_UI',{
	    'action':'AirboxRegs','subaction':'readSingle','dSID':dsid,'args':JSON.stringify(regs),
	});
    };

    this.writeRegs = function(dsid,regs) {
	return dss.Event.raise('wesco_UI',{
	    'action':'AirboxRegs','subaction':'writeSingle','dSID':dsid,'args':JSON.stringify(regs),
	});
    };

    this.writeRegs_toAll = function(regs) {
	return dss.Event.raise('wesco_UI',{
	    'action':'AirboxRegs','subaction':'writeAll','args':JSON.stringify(regs),
	});
    };


    this.storeConfigurableParameters = function() {//@Diego
	return dss.Event.raise('wesco_UI',{
	    'action':'StoredConfigurableParameters',
	});
    };

    this.storeExhaustFlap = function() {//@Diego
	return dss.Event.raise('wesco_UI',{
	    'action':'StoredExhaustFlap',
	});
    };

    this.restartAirboxes = function() {//@Diego
	return dss.Event.raise('wesco_UI',{
	    'action':'SetAirboxesLevel','level':'2',
	});
    };

    this.configVentilationSchedule = function(schedule) {//@Diego
	return dss.Event.raise('wesco_UI',{
	    'action':'configVentilationSchedule','schedule':JSON.stringify(schedule),
	});
    };


    this.clearAllAirboxErrors = function() {
	return dss.Event.raise('wesco_UI',{
	    'action':'ClearAllAirboxErrors',
	});
    };

    this.clearAllAirboxErrors = function() {
	return dss.Event.raise('wesco_UI',{
	    'action':'ClearAllAirboxErrors',
	});
    };

    this.clearAirboxError = function(dSUID) {
	return dss.Event.raise('wesco_UI',{
	    'action':'ClearAirboxError','dSUID':dSUID,
	});
    };

    this.getErrorLog = function() {
	return dss.getLogfile('/dss/js/wesco.log');
    };

    this.getConfig = function() {
	var result = {};
	var promises = [];
	promises.push(dss.Property.getString(rootpath+'/Notifications/PushTargets').then(function(value) {
	    result['PushTargets'] = JSON.parse(HTMLDecode(value));
	}));
	promises.push(dss.Property.getString(rootpath+'/Notifications/EmailAddress').then(function(value) {
	    result['EmailAddress'] = value;
	}));
	promises.push(dss.Property.getString(rootpath+'/ExhaustFlap').then(function(value) {
	    result['ExhaustFlap'] = value;
	}));
	promises.push(dss.Property.getInteger(rootpath+'/Controller/ClusterID').then(function(value) {
	    result['ClusterID'] = value;
	}));
	return $q.all(promises).then(function() {
	    return result;
	});
    };

    this.getAirboxConfig = function() {
	var result = {};
	var promises = [];
	promises.push(dss.Property.getInteger(rootpath+'/Airboxes/FilterReplacementInterval').then(function(value) {
	    result['FilterReplacementInterval'] = value;
	}));
	return $q.all(promises).then(function() {
	    return result;
	});
    };

    // Get info for a single airbox
    this.getAirbox = function(deviceid) {
	var result = {};
	var promises = [];

	    result['dSUID']=deviceid; // added variable dsuid (which is different than the dsid)!

	    promises.push(dss.Property.getString(rootpath+'/Airboxes/Devices/'+deviceid+'/dSID').then(function(value) {
		result['dSID'] = value;
		return dss.Device.getName(result['dSID']).then(function(name) {
		    result['Description'] = name;
		});
	    },function error(){
		result = {};
		return $q.reject();
	    }));

	    promises.push(dss.Property.getString(rootpath+'/Airboxes/Devices/'+deviceid+'/errorCode').then(function(value) {
		result['errorCode'] = value;
	    },function error(){
		result = {};
		return $q.reject("Field not found: errorCode");
	    }));

	    promises.push(dss.Property.getInteger(rootpath+'/Airboxes/Devices/'+deviceid+'/Status').then(function(value) {

		result['Status'] = "NORMAL";

		if(value & 0x01){
		result['Status'] = result['Status']+"LOCKED";
		}
		if (value & 0x40){
		    result['Status'] = result['Status']+"FAN_MISSING";
		}
		// console.log(value);
		// console.log(result['Status']);
	    },function error(){
		result = {};
		return $q.reject("Field not found: Status");
	    }));

	    promises.push(dss.Property.getString(rootpath+'/Airboxes/Devices/'+deviceid+'/errorTime').then(function(value) {
		result['errorTime'] = value;
	    },function error(){
		result = {};
		return $q.reject("Field not found: errorTime");
	    }));

	promises.push(dss.Property.getInteger(rootpath+'/Airboxes/Devices/'+deviceid+'/errorCounter').then(function(value) {
		result['errorCounter'] = value;
	},function error(){
	    result = {};
	    return $q.reject("Field not found: errorCounter");
	}));

	return $q.all(promises).then(function() {
	    return result;
	},function error (){
	    return $q.reject();
	});

    };

    // Get info for all configured airboxes
    this.getAirboxes = function() {
	var listReady = dss.Property.getChildren(rootpath+'/Airboxes/Devices')
	    .then(function(devices) {
		var devInfoPromises = devices.map(function(device) {
		    return service.getAirbox(device.name);
		});
		return $q.all(devInfoPromises);
	    });
	return listReady;
    };

    this.parseVarFile = function(contents) {
	var result = [];
	var lines = contents.split(/\n/);
	for(var i=0;i<lines.length;i++) {
	    if (!/\S/.test(lines[i])) {
		continue;
	    }
	    if (/#/.test(lines[i])) { // skip commentaren
		continue;
	    }

	    var data=lines[i].split(/\t/);
	    result.push({
		'reg':data[0],
		'value':data[1],
	    });
	}
	return result;
    }

}]);
